@@ -103,7 +103,7 @@ struct utimbuf
 };
 #endif
 
-#if defined(__GNUC__) && defined(AMIGA)
+#if defined(__GNUC__)
 /* gcc on the amiga need that __attribute((regparm)) must */
 /* be defined in function prototypes as well as in        */
 /* function definitions !                                 */
@@ -114,7 +114,7 @@ struct utimbuf
 
 /* sam: some definitions so that SAS/C can compile UAE */
 #if defined(__SASC) && defined(AMIGA)
-#define REGPARAM2 
+#define REGPARAM2
 #define REGPARAM
 #define S_IRUSR S_IREAD
 #define S_IWUSR S_IWRITE
@@ -277,7 +277,7 @@ extern void *xcalloc(size_t, size_t);
 
 #endif
 
-#endif /* _WIN32 */ 
+#endif /* _WIN32 */
 
 #ifdef DONT_HAVE_POSIX
 
@@ -387,7 +387,13 @@ extern int console_get (char *, int);
 #endif
 
 #ifndef STATIC_INLINE
+#if __GNUC__ - 1 > 1 && __GNUC_MINOR__ - 1 >= 0
+#define STATIC_INLINE static __inline__ __attribute__ ((always_inline))
+#define NOINLINE __attribute__ ((noinline))
+#else
 #define STATIC_INLINE static __inline__
+#define NOINLINE
+#endif
 #endif
 
 /* Every Amiga hardware clock cycle takes this many "virtual" cycles.  This
