@@ -15,7 +15,7 @@ extern void usage (void);
 extern void parse_cmdline (int argc, char **argv);
 
 
-extern void uae_reset (void);
+extern void uae_reset (int);
 extern void uae_quit (void);
 extern void reset_all_systems (void);
 
