@@ -37,6 +37,6 @@ void ViewPalette(int Color, int *Red, int *Green, int *Blue);
 void LoadPalette(void);
 int InitGfxLib(void);
 void CloseGfxLib(void);
-int SetMode(int ModeNumber, int Linear);
+int SetMode(int ModeNumber, int Linear, int Debug);
 void SaveMode(void);
 void RestoreMode(void);
