@@ -3,17 +3,10 @@
   *
   * Target specific stuff, SVGAlib version
   *
-  * Copyright 1997 Bernd Schmidt
+  * Copyright 1997, 1998 Bernd Schmidt
   */
 
-#define TARGET_SPECIAL_OPTIONS \
-    { "x",	  "  -x           : Don't use linear frame buffer even if it is available\n" },
-
-#define UNSUPPORTED_OPTION_l
-
-#ifndef USING_CURSES_UI
-#define UNSUPPORTED_OPTION_G
-#endif
+#define TARGET_NAME "svgalib"
 
 #define OPTIONSFILENAME ".uaerc"
 #define OPTIONS_IN_HOME
@@ -22,3 +15,5 @@
 #define DEFSERNAME "/dev/ttyS1"
 
 #define PICASSO96_SUPPORTED
+
+#define write_log write_log_standard
